package fachada;

import produto.Produto;
import produto.CadastroProdutos;
import produto.RepositorioProdutos;
import produto.RepositorioProdutosLista;

import funcionario.Funcionario;
import funcionario.CadastroFuncionarios;
import funcionario.RepositorioFuncionarios;
import funcionario.RepositorioFuncionariosLista;

import pedido.Pedido;
import pedido.CadastroPedidos;
import pedido.RepositorioPedidos;
import pedido.RepositorioPedidosLista;

import cliente.Cliente;
import cliente.CadastroClientes;
import cliente.RepositorioClientes;
import cliente.RepositorioClientesLista;

import util.DadosResidenciais;
import util.Iterator;


/**
 * Title: Classe Fachada
 * Description: fachada do sistema.
 * Copyright: Copyright (c) 2002
 * Company: VIP Team
 * @author VIP Team
 * @version 1.0
 */

public class Fachada {


    private CadastroProdutos produtos;
    private CadastroFuncionarios funcionarios;
    private CadastroPedidos pedidos;
    private CadastroClientes clientes;


    /**
     * Construtor padro.
     */
    public Fachada() {
/*        DadosResidenciais endereco1 = new DadosResidenciais("endereco1",
                                                            "telefone1",
                                                            "bairro1",
                                                            "referencia1",
                                                            "cidade1");
        Funcionario gerenteInicial = new Funcionario("Primeiro gerente",
                                                     endereco1,
                                                     5.0f,Funcionario.GERENTE,
                                                     "gerente","gerente");
*/        this.produtos = new CadastroProdutos(new RepositorioProdutosLista());
        this.funcionarios = new CadastroFuncionarios(
                                    new RepositorioFuncionariosLista());
//        this.funcionarios.inserir(gerenteInicial);
        this.pedidos = new CadastroPedidos(new RepositorioPedidosLista());
        this.clientes = new CadastroClientes(new RepositorioClientesLista());
    }

    /**
     * Checa se o repositorio nao est vazio.
     */
    public void touchFuncionario() throws exception.RepositorioFuncionariosVazioException {
        if (funcionarios.eVazio()) {
            throw new exception.RepositorioFuncionariosVazioException("Esta  a primeira vez q vc entra no sistema. Comece cadastrando o funcionrio inicial");
        }
    }

    /**
     * Faz a autenticao (login) do operador do sistema.
     *
     * @param login     nome do usurio.
     * @param senha     senha.
     * @return o objeto que representa esse funcionrio no sistema.
     */
    public Funcionario login(String login, String senha) {
        Funcionario f = new Funcionario();
        RepositorioFuncionarios r;
        Iterator iterador;
        Funcionario ret;

        f.setLogin(login);
        f.setSenha(senha);
        r = funcionarios.match(f,Funcionario.MATCH_LOGIN | Funcionario.MATCH_SENHA);
        iterador = r.getIterator();

        if (r.eVazio() == true) {
            throw new IllegalArgumentException("falha no login");
        } else {
            ret = (Funcionario) r.getIterator().next();
        }
        return ret;
    }

    /**
     * Troca a senha de um funcionrio. Apenas um GERENTE pode trocar a senha
     * de um funcionrio.
     *
     * @param f                         Funcionrio que ter a senha alterada.
     * @param novasenha                 Nova senha do funcionrio.
     * @param operador                  Operador logado
     */
     public void trocaSenha(Funcionario operador, Funcionario f, String novasenha) {
        RepositorioFuncionarios rep;
        Funcionario temp;
        Iterator iterador;

        rep = funcionarios.match(f, Funcionario.MATCH_LOGIN);
        iterador = rep.getIterator();
        temp = ((Funcionario) iterador.next());

        if (rep.eVazio()) {
            throw new IllegalArgumentException("Funcionrio no encontrado");
        }

        if (operador.getCargo() == Funcionario.GERENTE) {
            temp.setSenha(novasenha);
        } else if (operador.equals(f)) {
            if (f.getSenha().equals(temp.getSenha())) {
                temp.setSenha(novasenha);
            } else {
                throw new IllegalArgumentException("Senha antiga invlida");
            }
        } else {
            throw new IllegalArgumentException("Voc no tem permisso para " +
                                            "trocar a senha deste funcionario");
        }

     }

    /**
     * Cadastra um produto. Apenas um CAIXA pode cadastrar produtos.
     *
     * @param operador      Operador logado.
     * @param p             produto.
     */
    public void inserir(Funcionario operador, Produto p) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.CAIXA) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            produtos.inserir(p);
        }
    }

    /**
     * Descadastra um produto. Apenas um GERENTE pode remover um produto.
     *
     * @param operador      Operador logado
     * @param p             produto.
     */
    public void remover(Funcionario operador, Produto p) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.GERENTE) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            produtos.remover(p);
        }
    }

    /**
     * Cadastra um funcionrio. Apenas um GERENTE pode cadastrar um funcionrio.
     *
     * @param operador operador do sistema.
     * @param f funcionrio.
     */
    public void inserir(Funcionario operador, Funcionario f) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.GERENTE) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            funcionarios.inserir(f);
        }
    }

    /**
     * Cadastra o primeiro funcionario do sistema
     *
     * @param f funcionrio.
     */
    public void inserirInicial(Funcionario f) {
        funcionarios.inserir(f);
    }

    /**
     * Descadastra um funcionrio. Apenas um GERENTE pode descadastrar
     * um funcionrio.
     * Alm disso, um gerente NO pode se descadastrar.
     *
     * @param operador operador do sistema.
     * @param f funcionrio.
     */
    public void remover(Funcionario operador, Funcionario f) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.GERENTE) {
            throw new IllegalArgumentException("permisso negada");
        } else if (operador.equals(f) == true) {
            throw new IllegalArgumentException("voc no pode se descadastrar");
        } else {
            funcionarios.remover(f);
        }
    }

    /**
     * Cadastra um pedido. Apenas um CAIXA pode cadastrar um pedido. O cliente
     * a ser atendido por esse pedido DEVE estar cadastrado e TODOS os produtos
     * que constam no pedido tambm DEVEM estar cadastrados.
     *
     * @param operador operador do sistema.
     * @param p pedido.
     */
    public void inserir(Funcionario operador, Pedido p) {
        Iterator iterador;
        Produto temp;

        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.CAIXA) {
            throw new IllegalArgumentException("permisso negada");
        } else if (clientes.existe(p.getCliente()) == false) {
            throw new IllegalArgumentException("cliente no cadastrado");
        } else if (p.getFuncionario().equals(operador) == false) {
            throw new IllegalArgumentException("pedido tem que ser cadastrado "+
                                                "pelo operador");
        } else {
            iterador = p.getProdutos().getIterator();
            while (iterador.hasNext()) {
                temp = ((Produto) iterador.next());
                if (produtos.existe(temp) == false) {
                    throw new IllegalArgumentException("produto no est cadastrado => " + temp);
                }
            }
            pedidos.inserir(p);
        }
    }

    /**
     * Remove um pedido. Um GERENTE pode remover qualquer pedido, porm um CAIXA
     * s tem permisso de remover os pedidos que ele mesmo cadastrou.
     *
     * @param operador operador do sistema.
     * @param p pedido.
     */
    public void remover(Funcionario operador, Pedido p) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() == Funcionario.GERENTE) {
            pedidos.remover(p);
        } else if (operador.getCargo() == Funcionario.CAIXA
                   && p.getFuncionario().equals(operador) == false) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            pedidos.remover(p);
        }
    }

    /**
     * Cadastra um cliente. Apenas um CAIXA pode cadastrar clientes.
     *
     * @param operador operador do sistema.
     * @param c cliente.
     */
    public void inserir(Funcionario operador, Cliente c) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.CAIXA) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            clientes.inserir(c);
        }
    }

    /**
     * Descadastra um cliente. Apenas um CAIXA pode descadastrar clientes.
     *
     * @param operador operador do sistema.
     * @param c cliente.
     */
    public void remover(Funcionario operador, Cliente c) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.CAIXA) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            clientes.remover(c);
        }
    }


    /**
     * Atualiza as informaes de um cliente. Apenas um CAIXA pode faz-lo.
     *
     * @param operador operador do sistema.
     * @param c cliente.
     */
    public void atualizar(Funcionario operador, Cliente c) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.CAIXA) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            clientes.atualizar(c);
        }
    }

    /**
     * Atualiza as informaes de um produto. Apenas um CAIXA pode faz-lo.
     *
     * @param operador operador do sistema.
     * @param p produto.
     */
    public void atualizar(Funcionario operador, Produto p) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.CAIXA) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            produtos.atualizar(p);
        }
    }

    /**
     * Atualiza as informaes de um pedido. Apenas um CAIXA pode faz-lo.
     *
     * @param operador operador do sistema.
     * @param p pedido.
     */
    public void atualizar(Funcionario operador, Pedido p) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.CAIXA) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            pedidos.atualizar(p);
        }
    }

    /**
     * Atualiza as informaes de um funcionrio. Apenas um GERENTE pode faz-lo.
     *
     * @param operador operador do sistema.
     * @param f funcionario.
     */
    public void atualizar(Funcionario operador, Funcionario f) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.GERENTE) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            funcionarios.atualizar(f);
        }
    }

    /**
     * Retorna um RepositorioPedidos indicando os pedidos que casam
     * com o pedido fornecido de acordo com os parmetros passados.
     *
     * @param operador operador do sistema.
     * @param p         o pedido que ser comparado.
     *
     * @param criterios um inteiro indicando os campos (atributos) que sero
     *                  usados para comparar os pedidos.
     *
     *                  Os critrios vlidos so MATCH_NUMERO,
     *                  MATCH_NOME_CLIENTE e MATCH_NOME_FUNCIONARIO. Vrios
     *                  critrios podem ser utilizados ao mesmo tempo usando OU
     *                  lgico.
     *
     * @return          RepositorioPedidos com os Pedidos que casam
     *                  com o dado.
     *
     * @see Funcionario
     * @see Lista
     */
    public RepositorioPedidos match(Funcionario operador, Pedido p,
                                    int criterios) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else {
            return pedidos.match(p,criterios);
        }
    }

    /**
     * Retorna uma lista de produtos que atendem aos critrios definidos.
     *
     * @param operador operador do sistema.
     * @param p         o produto que ser usado como base da comparao.
     * @param criterios  um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o produto base com os produtos
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_DESCRICAO,
     *                  MATCH_TAMANHO, MATCH_PRECO.
     *
     * @return um repositorio de produtos que atendem aos critrios definidos.
     * @see Produto
     * @see Lista
     */
    public RepositorioProdutos match(Funcionario operador, Produto p,
                                    int criterios) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else {
            return produtos.match(p,criterios);
        }
    }


    /**
     * Retorna um repositrio com os clientes que atendem aos critrios definidos.
     *
     * @param operador operador do sistema.
     * @param c          o cliente que ser usado como base da comparao.
     * @param criterios  um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o cliente base com os clientes
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_BAIRRO.
     * @return          um repositrio com os clientes achados.
     */
    public RepositorioClientes match(Funcionario operador, Cliente c,
                                    int criterios) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else {
            return clientes.match(c,criterios);
        }
    }


    /**
     * Retorna um RepositorioFuncionarios indicando os funcionrios que casam
     * com o funcionario fornecido de acordo com os parmetros passados. Apenas
     * um GERENTE pode executar essa operao.
     *
     * @param operador operador do sistema.
     * @param f         o funcionrio que ser comparado.
     *
     * @param criterios um inteiro indicando os campos (atributos) que sero
     *                  usados para comparar os funcionrios.
     *
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_CIDADE, MATCH_BAIRRO, MATCH_CARGO, MATCH_LOGIN e
     *                  MATCH_SENHA. Vrios critrios podem
     *                  ser utilizados ao mesmo tempo usando OU lgico.
     *
     * @return          RepositorioFuncionarios com os Funcionrios que casam
     *                  com o dado.
     *
     * @see Funcionario
     * @see Lista
     *
     */
    public RepositorioFuncionarios match(Funcionario operador, Funcionario f,
                                        int criterios) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.GERENTE) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            return funcionarios.match(f,criterios);
        }
    }

    /**
     * Retorna um booleano indicando se um funcionrio faz um <i>like</i>
     * com outro.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @return          um boolean que diz se os funcionrios casam.
     *
    */
    public RepositorioFuncionarios like(Funcionario operador, Funcionario f) {
        if (operador == null) {
            throw new IllegalArgumentException("operador no logado");
        } else if (operador.getCargo() != Funcionario.GERENTE) {
            throw new IllegalArgumentException("permisso negada");
        } else {
            return funcionarios.like(f);
        }
    }

    public static void main(String[] args) {
        Fachada fachada = new Fachada();
        DadosResidenciais dr1 = new DadosResidenciais("Rua 1","1111-1111",
                                                      "bairro 1","perto de 2",
                                                      "cidade 1");
        DadosResidenciais dr2 = new DadosResidenciais("Rua 2","2222-2222",
                                                      "bairro 2","perto de 1",
                                                      "cidade 2");
        Cliente c1 = new Cliente("Cliente 1",dr1);
        Cliente c2 = new Cliente("Cliente 2",dr2);
        Funcionario f1 = new Funcionario("joao",dr2,200,Funcionario.CAIXA,
                                        "joao","joao");
        Produto p5 = new Produto("Coca-Cola","Coca-Cola",Produto.TAMANH0_LATA,
                                  1.0f);

        // vai gerar uma excecao pois o operador no est logado:
//        fachada.inserirCliente(c1);

        // senha errada:
//        fachada.login("gerente","errada");


        // gerente no pode cadastrar clientes:
//        fachada.inserirCliente(c2);

        // gerente cadastrando um funcionario:
        fachada.inserir(f1, f1);

        // gerente sai do sistema:
//        fachada.logout();

        // funcionario joao entra:
        fachada.login("joao","joao");

        // joao cadastra um produto:
        fachada.inserir(f1, p5);

        // joao tenta cadastrar o MESMO produto 2x:
//        fachada.inserirProduto(Produto.COCACOLA);

        // joao cadastra um cliente:
        fachada.inserir(f1, c1);

        // joao tenta cadastrar um pedido cujo cliente NAO est cadastrado:
        RepositorioProdutos prods = new RepositorioProdutosLista();
        Pedido p1 = new Pedido(0,c2,f1 ,prods,2.5f);
//        fachada.inserirPedido(p1);

        // joao troca o cliente para um j cadastrado:
        p1.setCliente(c1);

        // o problema agora  que a lista de produtos do pedido NAO pode ser
        // vazia:
//        fachada.inserirPedido(p1);

        // inserir um produto NAO cadastrado na lista de produtos e depois
        // tentar cadastrar esse pedido:
//        p1.getProdutos().inserir(Produto.CALABREZA);
//        fachada.inserirPedido(p1);

        // dessa vez tudo ok:
        p1.getProdutos().inserir(p5);
        fachada.inserir(f1, p1);

        // lista os pedidos cadastrados:
        System.out.println(fachada.pedidos);

        // lista os produtos cadastrados:
        System.out.println(fachada.produtos);
    }

}