package cliente;

import util.DadosResidenciais;

import exception.ClienteJaCadastradoException;
import exception.ClienteNaoCadastradoException;

/**
 * <p>Title: CadastroClientes</p>
 * <p>Description: Classe que controla o cadastro de clientes</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * @author Rafael Cavalcanti Ortolan
 * @version 1.0
 */

public class CadastroClientes {


    private RepositorioClientes clientes;

    /**
     * Construtor.
     */
    public CadastroClientes(RepositorioClientes clientes) {
        this.clientes = clientes;
    }

    /**
     * Lista os clientes cadastrados.
     *
     * @return      uma String que representa o cadastro.
     */
    public String toString() {
        return this.clientes.toString();
    }

    /**
     * Insere um cliente <b>novo</b> no cadastro.
     *
     * @param cliente     novo cliente.
     */
    public void inserir(Cliente cliente) throws ClienteJaCadastradoException {
        if (this.existe(cliente) == true) {
            throw new ClienteJaCadastradoException();
        } else {
            clientes.inserir(cliente);
        }
    }

    /**
     * Verifica se um cliente existe no cadastro.
     *
     * @param cliente   cliente a ser verificado.
     * @return          um booleano indicando se o cliente est cadastrado.
     */
    public boolean existe(Cliente cliente) {
        return clientes.existe(cliente);
    }

    /**
     * Remove um cliente do cadastro.
     *
     * @param cliente     cliente a ser removido.
     */
    public void remover(Cliente cliente) throws ClienteNaoCadastradoException{
        if (this.existe(cliente) == false) {
            throw new ClienteNaoCadastradoException(cliente);
        } else {
            clientes.remover(cliente);
        }
    }

    /**
     * Retorna um repositorio de clientes que atendem aos critrios definidos.
     *
     * @param c         o cliente que ser usado como base da comparao.
     * @param criterios um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o cliente base com os clientes
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_BAIRRO.
     * @return          um repositorio com todos os clientes encontrados
     *                  que fazem <i>match</i> com o cliente base segundo os
     *                  critrios especificados.
     *
     * @see Cliente
     * @see RepositorioClientes
     */
    public RepositorioClientes match(Cliente c, int criterios) {
        return this.clientes.match(c, criterios);
    }

    /**
     * Atualiza um cliente no repositorio
     *
     * @param c     cliente que ser atualizado.
     */
    public void atualizar(Cliente c) {
        this.clientes.atualizar(c);
    }

    public static void main(String[] args) {
        boolean ok;
        DadosResidenciais endereco = new DadosResidenciais("lal","lele"
            ,"lil","lol","lul");
        Cliente cliente = new Cliente("Rafael",endereco);
        Cliente m = new Cliente("Rafael",endereco);
        CadastroClientes teste = new CadastroClientes(
            new RepositorioClientesLista());
        RepositorioClientes rep;

//        teste.inserir(cliente);
        System.out.print(teste.existe(cliente));
        rep = teste .match(m,Cliente.MATCH_NOME);
//        teste.remover(cliente);
//        System.out.print(teste.existe(cliente));
        System.out.print(rep.toString());
        DadosResidenciais endereco2 = new DadosResidenciais("lal"
            ,"lele","lil","lol","lul");
        Cliente cliente2 = new Cliente("Gustavo",endereco2);
//        teste.inserir(cliente);
        System.out.print(teste.existe(cliente));
    }

}