package cliente;

import pessoa.Pessoa;
import util.DadosResidenciais;

/**
 * <p>Title: Cliente</p>
 * <p>Description: define a classe Cliente.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * @author Rafael Cavalcanti Ortolan
 * @version 1.0
 */


public class Cliente extends Pessoa {


    // constantes de match
    public final static int MATCH_NOME = 1;
    public final static int MATCH_TELEFONE = 2;
    public final static int MATCH_BAIRRO = 4;
    public final static int MATCH_CIDADE = 8;
    public final static int MATCH_ENDERECO = 16;

    /**
     * Construtor padro da classe Cliente.
     *
     * Constri um cliente padro.
     */
    public Cliente() {
        super();
    }


    /**
     * Construtor da classe Cliente.
     *
     * @param nome                  nome do cliente.
     * @param dadosResidenciais     os dados residenciais do cliente.
     */
    public Cliente(String nome, DadosResidenciais dadosResidenciais) {
        super (nome, dadosResidenciais);
    }

    /**
     * Retorna uma string que representa o cliente.
     *
     * @return              uma string com o nome do cliente seguido de seu
     *                      telefone entre parntesis.
     */
    public String toString() {
        return getNome() + " (" + getDadosResidenciais().getTelefone() + ")";
    }

    /**
     * Compara dois clientes.
     *
     * Os critrios de comparao utilizados so o <b>nome</b> e o
     * <b>telefone</b>.
     *
     * @param c     cliente a ser comparado.
     * @return      um booleano indicando se os clientes so equivalentes.
     */
    public boolean equals(Object c) {
        boolean retorno = false;

        if (c instanceof Cliente) {
            if (this.getNome().equals(((Cliente) c).getNome())
                    || this.getDadosResidenciais().getTelefone().
                    equals(((Cliente) c).getDadosResidenciais().
                    getTelefone())) {
                retorno = true;
            }
        } else {
            throw new IllegalArgumentException("Pessoa no  do tipo Cliente");
        }

        return retorno;
    }

    /**
     * Retorna um booleano indicando se um cliente faz um <i>match</i> com outro
     * de acordo com os critrios desejados.
     *
     * @param c         o cliente que ser comparado.
     * @param criterio  um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar esse cliente (<i>this</i>) com o
     *                  cliente fornecido.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_BAIRRO.
     *
     * @return um booleano indicando se os clientes fazem <i>match</i>.
     */
    public boolean match(Pessoa c, int criterios) {
        boolean retorno = true;

        if (c instanceof Cliente) {
            if (((criterios & MATCH_NOME) == MATCH_NOME) &&
                    !(getNome().equalsIgnoreCase(((Cliente) c).getNome())  )) {
                retorno = false;
            }

            if ((criterios & MATCH_TELEFONE) == MATCH_TELEFONE &&
                    !(getDadosResidenciais().getTelefone().equalsIgnoreCase(((Cliente) c).
                    getDadosResidenciais().getTelefone()))) {
                retorno = false;
            }

            if ((criterios & MATCH_BAIRRO) == MATCH_BAIRRO &&
                    !(getDadosResidenciais().getBairro().equalsIgnoreCase(((Cliente) c).
                    getDadosResidenciais().getBairro()))) {
                retorno = false;
            }
            if ((criterios & MATCH_CIDADE) == MATCH_CIDADE &&
                    !(getDadosResidenciais().getCidade().equalsIgnoreCase(((Cliente) c).
                    getDadosResidenciais().getCidade()))) {
                retorno = false;
            }
            if ((criterios & MATCH_ENDERECO) == MATCH_ENDERECO &&
                    !(getDadosResidenciais().getEndereco().equalsIgnoreCase(((Cliente) c).
                    getDadosResidenciais().getEndereco()))) {
                retorno = false;
            }
        } else {
            throw new IllegalArgumentException("Pessoa no  do tipo Cliente");
        }

        return retorno;
    }

     /**
     * Atualiza um cliente
     *
     * @param c     cliente que ser atualizado.
     */
    public void atualizar(Pessoa c) {
        super.atualizar(c);
    }

}