package cliente;

import util.Iterator;

/**
 * <p>Title: RepositorioClientes</p>
 * <p>Description: Interface para o repositrio de clientes</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * @author Rafael Cavalcanti Ortolan
 * @version 1.0
 */

public interface RepositorioClientes {

     /**
     *   Este metodo coloca inseri cliente
     *
     * @param cliente    Sera utilizado para inclusao
     */
    public void inserir(Cliente cliente);

     /**
     *   Este metodo remove um cliente
     *
     * @param cliente    Sera utilizado para remocao
     */
    public void remover(Cliente cliente);

    /**
     * Verifica se um cliente existe
     *
     * @param cliente cliente
     * @return  um booleano indicando a existncia do cliente.
     */
    public boolean existe(Cliente cliente);

    /**
     * Retorna um array que representa o repositrio.
     *
     * @return um array com os clientes do repositrio.
     */
//    public Cliente[] toArray();

     /**
     * Verifica se o repositrio  vazio.
     *
     * @return um boolean indicando se o repositrio  vazio.
     */
    public boolean eVazio();

     /**
     * Atualiza um cliente no repositrio.
     *
     * @param c     cliente que ser atualizado.
     */
    public void atualizar(Cliente c);

    /**
     * Retorna um repositrio com os clientes que atendem aos critrios
     * definidos.
     *
     * @param c          o cliente que ser usado como base da comparao.
     * @param criterios  um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o cliente base com os clientes
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_BAIRRO.
     * @return          um repositrio com os clientes achados.
     */
    public RepositorioClientes match(Cliente c, int criterios);

    /**
     * Retorna um iterador para a classe RepositorioClientes
     *
     * @return    um iterador.
     */
    public Iterator getIterator();
}