package cliente;

import util.Lista;
import util.DadosResidenciais;
import util.Iterator;
import util.IteratorArray;

/**
 * <p>Title: ListaClientes</p>
 * <p>Description: Implementa uma lista de clientes</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * @author Rafael Cavalcanti Ortolan
 * @version 1.0
 */

public class RepositorioClientesLista implements RepositorioClientes {


     private Lista clientes;

    /**
     * Construtor padro.
     */
    public RepositorioClientesLista() {
        this.clientes = new Lista();
    }

    /**
     * Insere um cliente na lista.
     *
     * @param cliente         cliente a ser inserido na lista.
     */
    public void inserir(Cliente cliente) {
        clientes.inserir(cliente);
    }

    /**
     * Remove um cliente da lista. Tal cliente <b>deve</b> constar na lista.
     *
     * @param cliente         cliente a ser removido.
     */
    public void remover(Cliente cliente) {
        clientes.remover(cliente);
    }

    /**
     * Verifica se um cliente existe na lista.
     *
     * @param cliente   cliente.
     * @return          um booleano indicando a presena do cliente na lista.
     */
    public boolean existe(Cliente cliente) {
        return clientes.existe(cliente);
    }

    /**
     * Retorna um repositrio com os clientes que atendem aos critrios
     * definidos.
     *
     * @param c          o cliente que ser usado como base da comparao.
     * @param criterios  um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o cliente base com os clientes
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_BAIRRO.
     * @return          um repositrio com os clientes achados.
     */
    public RepositorioClientes match(Cliente c, int campos) {
        RepositorioClientes retorno;
        Cliente temp;
//        Cliente[] array = this.toArray();
//        int i;
        Iterator iterador = getIterator();

        retorno = new RepositorioClientesLista();


        while (iterador.hasNext()) {
            temp = (Cliente) iterador.next();
            if (c.match(temp,campos)) {
                retorno.inserir(temp);
            }
        }
/*        for (i = 0; i < array.length; i++) {
            if (c.match(array[i],campos)) {
                retorno.inserir(array[i]);
            }
        } */

        return retorno;
    }

    /**
     * Retorna os clientes contidos nessa lista.
     *
     * @return     "[]" caso a lista seja vazia. Caso contrrio retorna uma
     *             lista que comea com '[' e termina com ']' em que os
     *             elementos so separados por vrgula.
     * @see Cliente
     */
    public String toString() {
        return clientes.toString();
    }

    /**
     * Retorna um array que representa a lista.
     *
     * @return um array com os clientes da lista.
     */
/*    public Cliente[] toArray() {
        int i;
        Object[] array = clientes.toArray();
        Cliente[] ret = new Cliente[array.length];

        for (i = 0; i < array.length; i++) {
            ret[i] = (Cliente) array[i];
        }

        return ret;
    } */

     /**
     * Verifica se a lista de clientes  vazia.
     *
     * @return um boolean indicando se a lista  vazia.
     */
    public boolean eVazio() {
        return this.clientes.eVazio();
    }

     /**
     * Atualiza um cliente no repositorio
     *
     * @param c     cliente que ser atualizado.
     */
    public void atualizar(Cliente c) {
        RepositorioClientes rep;
//        Cliente[] arr;
        Iterator iterador;

        rep = this.match(c,Cliente.MATCH_NOME | Cliente.MATCH_TELEFONE);

        if (rep.eVazio() == true) {
            throw new IllegalArgumentException("cliente no encontrado");
        } else {
            iterador = rep.getIterator();
            ((Cliente) iterador.next()).atualizar(c);
//            arr = rep.toArray();
//            arr[0].atualizar(c);
        }
    }

    /**
     * Retorna um iterador para a classe RepositorioClientesLista
     *
     * @return    um iterador.
     */
    public Iterator getIterator() {
        return new IteratorArray(clientes);
    }

    public static void main(String[] args) {
        boolean b;
        RepositorioClientesLista lista = new RepositorioClientesLista();
        DadosResidenciais dados1 = new DadosResidenciais("endereco1"
            ,"telefone1","bairro1","referencia1","cidade1");
        DadosResidenciais dados2 = new DadosResidenciais("endereco2"
            ,"telefone2","bairro2","referencia2","cidade2");
        DadosResidenciais dados3 = new DadosResidenciais("endereco3"
            ,"telefone3","bairro3","referencia3","cidade3");
        Cliente c1 = new Cliente("Rafael",dados1);
        Cliente c2 = new Cliente("Thiago",dados2);
        Cliente c3 = new Cliente("Neto",dados3);
        Cliente c4 = new Cliente("Thiago",dados3);

        lista.inserir(c1);
        System.out.println(lista);
        lista.inserir(c2);
        System.out.println(lista);
        lista.inserir(c1);
        System.out.println(lista);
        System.out.println(lista.match(c4, Cliente.MATCH_BAIRRO));
        System.out.println(c4.getDadosResidenciais().getBairro().
            equals(c2.getDadosResidenciais().getBairro()));
        System.out.println(c2.getDadosResidenciais().getBairro());
    }
}