package funcionario;

import util.DadosResidenciais;


/**
 * Title: Classe CadastroFuncionarios
 * Description: Negcio
 * Copyright:    Copyright (c) 2002
 * Company: VIP Team
 * @author Thiago Arajo
 * @version 1.0
 */

public class CadastroFuncionarios {


    private RepositorioFuncionarios funcionarios;

    /**
     * Construtor.
     */
    public CadastroFuncionarios(RepositorioFuncionarios funcionarios) {
        this.funcionarios = funcionarios;
    }

    /**
     * Checa se o repositorio nao est vazio.
     */
    public boolean eVazio() {
        return funcionarios.eVazio();
    }

    /**
     * Insere um funcionrio <b>novo</b> no cadastro.
     *
     * @param funcionario     novo funcionrio.
     */
    public void inserir(Funcionario funcionario) {
        boolean resposta;

        resposta = funcionarios.existe(funcionario);
        if (resposta) {
            throw new IllegalArgumentException("funcionrio j cadastrado");
        } else {
            funcionarios.inserir(funcionario);
        }
    }

    /**
     * Remove um funcionrio do cadastro.
     *
     * @param funcionario     funcionrio a ser removido.
     */
    public void remover(Funcionario funcionario) {
        funcionarios.remover(funcionario);
    }

    /**
     * Verifica se um funcionrio existe no cadastro.
     *
     * @param funcionario     funcionrio a ser verificado.
     * @return      um booleano indicando se o funcionrio est cadastrado.
     */
    public boolean existe(Funcionario funcionario) {
        return funcionarios.existe(funcionario);
    }

    /**
     * Lista os funcionrios cadastrados.
     *
     * @return      uma String que representa o cadastro.
     */
    public String toString() {
        return this.funcionarios.toString();
    }

    /**
     * Retorna um array que representa o repositrio.
     *
     * @return um array com os funcionrios do repositrio.
    */
/*    public Funcionario[] toArray() {
        return funcionarios.toArray();
    } */

    /**
     * Retorna um RepositorioFuncionarios indicando os funcionrios que casam
     * com o funcionario fornecido de acordo com os parmetros passados.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @param campos    um inteiro indicando os campos (atributos) que sero
     *                  usados para comparar esse funcionrio (<i>this</i>) com
     *                  o funcionrio fornecido.
     *
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_CIDADE, MATCH_BAIRRO, MATCH_CARGO, MATCH_LOGIN e
     *                  MATCH_SENHA. Vrios critrios podem
     *                  ser utilizados ao mesmo tempo usando OU lgico.
     *
     * @return          RepositorioFuncionarios com os Funcionrios que casam
     *                  com o dado.
     *
     * @see Funcionario
     * @see Lista
     *
    */
    public RepositorioFuncionarios match(Funcionario f, int criterios) {
        return this.funcionarios.match(f,criterios);
    }

    /**
     * Retorna um booleano indicando se um funcionrio faz um <i>like</i>
     * com outro.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @return          um boolean que diz se os funcionrios casam.
     *
    */
    public RepositorioFuncionarios like(Funcionario f) {
        return this.funcionarios.like(f);
    }

    /**
     * Atualiza um funcionrio no repositorio
     *
     * @param f     funcionrio que ser atualizado.
     */
    public void atualizar(Funcionario f) {
        this.funcionarios.atualizar(f);
    }

    public static void main(String[] args) {
        DadosResidenciais d = new DadosResidenciais("Av. Boa Viagem 914,"
        + " apto 901","33267058","Boa Viagem","Vizinho ao meu vizinho","Recife");

        DadosResidenciais e = new DadosResidenciais("Av. Boa Viagem 914,"
        + " apto 901","34658753","Boa Viagem","Vizinho ao meu vizinho","Recife");

        Funcionario f = new Funcionario("Thiago", d, 10000, Funcionario.GERENTE, "thiago", "senha1");
        Funcionario g = new Funcionario("Rafael", d, 100, Funcionario.COZINHEIRO, "rafael", "senha1");
        Funcionario h = new Funcionario("Neto", e, 10, Funcionario.COZINHEIRO, "neto", "senha1");
        Funcionario i = new Funcionario("Rafael", e, 10, Funcionario.COZINHEIRO, "rafael", "senha1");
        RepositorioFuncionarios l;

        CadastroFuncionarios cad = new CadastroFuncionarios(new RepositorioFuncionariosLista());

        cad.inserir(f);
        cad.inserir(g);
        cad.inserir(h);
//        cad.inserir(i);

        l = cad.funcionarios.match(i, Funcionario.MATCH_LOGIN);
    }
}

