package funcionario;

import pessoa.Pessoa;
import util.DadosResidenciais;

/**
 * Title: Classe Funcionario
 * Description: Negcio Bsico
 * Copyright:    Copyright (c) 2002
 * Company: VIP Team
 * @author Thiago Arajo
 * @version 1.0
 */


public class Funcionario extends Pessoa {

    // constantes de tamanho.
    public static final int CAIXA = 0;
    public static final int COZINHEIRO = 1;
    public static final int ENTREGADOR = 2;
    public static final int GERENTE = 3;
    public static final int CARGO_VAZIO = 4;


    // constantes de match.
    public static final int MATCH_NOME = 1;
    public static final int MATCH_TELEFONE = 2;
    public static final int MATCH_CIDADE = 3;
    public static final int MATCH_BAIRRO = 4;
    public static final int MATCH_CARGO = 8;
    public static final int MATCH_LOGIN = 16;
    public static final int MATCH_SENHA = 32;

    private float salario;
    private int cargo;
    private String login;
    private String senha;

    /**
     * Construtor padro da classe Funcionrio.
     *
     */
    public Funcionario() {
        super();
        salario = 0;
        cargo = CARGO_VAZIO;
        login = new String("");
        senha = new String("");
    }

     /**
     * Construtor da classe Produto.
     *
     * @param nome                  Nome do funcionario.
     * @param dadosresidenciais     Os dados residenciais do funcionrio.
     * @param salario               Salrio do funcionrio.
     * @param cargo                 Constante que indica o cargo.
     * @param login                 Login do funcionrio.
     * @param senha                 Senha do funcionrio.
     */
    public Funcionario(String nome, DadosResidenciais dadosResidenciais,
        float salario, int cargo, String login, String senha)
        throws IllegalArgumentException  {

        super (nome, dadosResidenciais);
        setSalario(salario);
        setCargo(cargo);
        setLogin(login);
        setSenha(senha);
    }

    /**
     * Retorna o salrio do funcionrio.
     *
     * @return              float contendo o salrio do funcionrio.
    */
    public float getSalario() {
        return salario;
    }

    /**
     * Altera o salrio do funcionrio.
     *
     * @param salario     novo salrio do funcionrio.
    */
    public void setSalario(float salario) {
        if (salario >= 0) {
            this.salario = salario;
        } else {
            throw new IllegalArgumentException("Salrio no pode ser vazio");
        }
    }

    /**
     * Retorna o cargo do funcionrio.
     *
     * @return              int q representa o cargo do funcionrio.
    */
    public int getCargo() {
        return cargo;
    }

    /**
     * Altera o cargo do funcionrio.
     *
     * @param salario     int com o novo cargo do funcionrio.
    */
    public void setCargo(int cargo) {
        if (cargo == COZINHEIRO || cargo == GERENTE  || cargo == CAIXA ||
                cargo == ENTREGADOR || cargo == CARGO_VAZIO) {
            this.cargo = cargo;
        } else {
            throw new IllegalArgumentException("Cargo Invlido");
        }
    }

    public String getCargoDescricao() {
        switch (cargo) {
            case CAIXA:
                return "Caixa";
            case COZINHEIRO:
                return  "Cozinheiro";
            case ENTREGADOR:
                return   "Entregador";
            case GERENTE:
                return "Gerente";
            default:
                return "n/a";
        }
    }

    /**
     * Retorna o login do funcionrio.
     *
     * @return              String com o o login do funcionrio.
    */
    public String getLogin() {
        return this.login;
    }

    /**
     * Altera o login do funcionrio.
     *
     * @param salario     String com o novo login do funcionrio.
    */
    public void setLogin(String login) {
        if (!login.equals("")) {
            this.login = login;
        } else {
            throw new IllegalArgumentException("Login invlido");
        }
    }

    /**
     * Retorna a senha do funcionrio.
     *
     * @return              String com a senha do funcionrio.
    */
    public String getSenha() {
        return this.senha;
    }

    /**
     * Altera a senha do funcionrio.
     *
     * @param salario     String com a nova senha do funcionrio.
    */
    public void setSenha(String senha) {
        if (!senha.equals("")) {
            this.senha = senha;
        } else {
            throw new IllegalArgumentException("Senha invlida");
        }
    }

    /**
     * Compara dois funcionrios.
     *
     * Os critrios de comparao utilizados so o nome e o
     * cargo.
     *
     * @param f             funcionrio a ser comparado.
     * @return              Boolean indicando se os funcionrios so iguais.
     */
    public boolean equals(Object o) {
        boolean resp;
        Pessoa f;
        if (o instanceof Funcionario) {
            f = (Funcionario) o;
            if (f instanceof Funcionario) {
                if (this.login.equals(((Funcionario) f).login)) {
                    resp = true;
                } else {
                    resp = false;
                }
            } else {
                throw new IllegalArgumentException("pessoa no  funcionrio");
            }
        } else {
            throw new IllegalArgumentException("objeto no  funcionrio");
        }
        return resp;
    }

     /** Atualiza um funcionrio
     *
     * @param f     funcionrio que ser atualizado.
     */
    public void atualizar(Pessoa p) throws IllegalArgumentException {
        Funcionario f;

        if (p instanceof Funcionario) {
            f = (Funcionario) p;
            super.atualizar(f);
            setCargo(f.cargo);
            setSenha(f.senha);
            setSalario(f.salario);
        } else {
            throw new IllegalArgumentException("pessoa no  funcionrio");
        }
    }

    /**
     * Retorna uma String que representa o funcionrio.
     *
     * @return              uma String com o nome do funcionrio.
     */
    public String toString() {
        return getNome();
    }

    /**
     * Retorna um booleano indicando se um funcionrio faz um <i>match</i>
     * com outro de acordo com os critrios desejados.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @param campos    um inteiro indicando os campos (atributos) que sero
     *                  usados para comparar esse funcionrio (<i>this</i>) com
     *                  o funcionrio fornecido.
     *
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_CIDADE, MATCH_BAIRRO, MATCH_CARGO, MATCH_LOGIN e
     *                  MATCH_SENHA. Vrios critrios podem
     *                  ser utilizados ao mesmo tempo usando OU lgico.
     *
     * @return          um boolean que diz se os funcionrios casam de acordo
     *                  com os paramentros.
     *
    */
    public boolean match(Pessoa p, int campos) {
        boolean ok;
        Funcionario f;
        ok = true;

        ok = true;
        if (p instanceof Funcionario) {
            f = (Funcionario) p;
            if ((campos & MATCH_NOME) == MATCH_NOME) {
                if (!f.getNome().equals(super.getNome())) {
                    ok = false;
                }
            }
            if ((campos & MATCH_TELEFONE) == MATCH_TELEFONE) {
                if (!f.getDadosResidenciais().getTelefone().
                    equals(super.getDadosResidenciais().getTelefone())) {
                    ok = false;
                }
            }
            if ((campos & MATCH_BAIRRO) == MATCH_BAIRRO) {
                if (!f.getDadosResidenciais().getBairro()
                    .equals(super.getDadosResidenciais().getBairro())) {
                    ok = false;
                }
            }
            if ((campos & MATCH_CIDADE) == MATCH_CIDADE) {
                if (!f.getDadosResidenciais().getCidade()
                    .equals(super.getDadosResidenciais().getCidade())) {
                    ok = false;
                }
            }
            if ((campos & MATCH_CARGO) == MATCH_CARGO) {
                if (f.getCargo() != cargo) {
                    ok = false;
                }
            }
            if ((campos & MATCH_LOGIN) == MATCH_LOGIN) {
                if (!f.getLogin().equals(login)) {
                    ok = false;
                }
            }
            if ((campos & MATCH_SENHA) == MATCH_SENHA) {
                if (!f.getSenha().equals(senha)) {
                    ok = false;
                }
            }
        } else {
            throw new IllegalArgumentException("pessoa no  funcionrio");
        }
        return ok;
    }

    /**
     * Retorna um booleano indicando se um funcionrio faz um <i>like</i>
     * com outro.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @return          um boolean que diz se os funcionrios casam.
     *
    */
    public boolean like(Pessoa p) {
        boolean ok;
        Funcionario f;
        ok = true;

        if (p instanceof Funcionario) {
            f = (Funcionario) p;
            if  ((super.getNome().toLowerCase().indexOf(f.getNome().toLowerCase()) == -1) ||
                    (super.getDadosResidenciais().getTelefone().toLowerCase().indexOf(f.getDadosResidenciais().getTelefone().toLowerCase()) == -1) ||
                    (getCargo() != f.getCargo() &&
                    f.getCargo() != CARGO_VAZIO) ||
                    (super.getDadosResidenciais().getBairro().toLowerCase().indexOf(f.getDadosResidenciais().getBairro().toLowerCase()) == -1) ||
                    (super.getDadosResidenciais().getCidade().toLowerCase().indexOf(f.getDadosResidenciais().getCidade().toLowerCase()) == -1) ||
                    (getLogin().toLowerCase().indexOf(f.getLogin().toLowerCase()) == -1)) {
                ok = false;
            }
        } else {
            throw new IllegalArgumentException("pessoa no  funcionrio");
        }
        return ok;
    }
}