package funcionario;

import util.Iterator;

/**
 * Title: Classe RepositorioFuncionarios
 * Description: Camada de Dados
 * Copyright:    Copyright (c) 2002
 * Company: VIP Team
 * @author Thiago Arajo
 * @version 1.0
 */

public interface RepositorioFuncionarios {

    /**
     * Insere um funcionrio.
     *
     * @param funcionario         funcionario a ser inserido.
     */
    public void inserir(Funcionario funcionario);

    /**
     * Remove um funcionrio.
     *
     * @param funcionario         funcionario a ser removido.
    */
    public void remover(Funcionario funcionario);

    /**
     * Verifica se um funcionrio existe na lista.
     *
     * @param funcionrio         funcionario.
     * @return                    um boolean indicando a presena do funcionrio.
    */
    public boolean existe(Funcionario funcionario);

    /**
     * Retorna os funcionrios contidos nessa lista.
     *
     * @return     "[]" caso a lista seja vazia. Caso contrrio retorna uma
     *             lista que comea com '[' e termina com ']' em que os
     *             elementos so separados por vrgula.
     * @see Funcionario
     */
    public String toString();

    /**
     * Retorna um RepositorioFuncionarios indicando os funcionrios que casam
     * com o funcionario fornecido de acordo com os parmetros passados.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @param campos    um inteiro indicando os campos (atributos) que sero
     *                  usados para comparar esse funcionrio (<i>this</i>) com
     *                  o funcionrio fornecido.
     *
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_CIDADE, MATCH_BAIRRO, MATCH_CARGO, MATCH_LOGIN e
     *                  MATCH_SENHA. Vrios critrios podem
     *                  ser utilizados ao mesmo tempo usando OU lgico.
     *
     * @return          RepositorioFuncionarios com os Funcionrios que casam
     *                  com o dado.
     *
     * @see Funcionario
     * @see Lista
     *
    */
    public RepositorioFuncionarios match(Funcionario f, int campos);

    /**
     * Retorna um booleano indicando se um funcionrio faz um <i>like</i>
     * com outro.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @return          um boolean que diz se os funcionrios casam.
     *
    */
    public RepositorioFuncionarios like(Funcionario f);

    /**
     * Verifica se o repositrio  vazio.
     *
     * @return um boolean indicando se o repositrio  vazio.
     */
    public boolean eVazio();

    /**
     * Atualiza um funcionario no repositorio
     *
     * @param f     funcionario que ser atualizado.
     */
    public void atualizar(Funcionario f);

    /**
     * Retorna um iterador para a classe RepositorioFuncionarios
     *
     * @return    um iterador.
     */
    public Iterator getIterator();
}