package funcionario;

import util.Lista;
import util.Iterator;
import util.IteratorArray;

/**
 * Title: Classe ListaFuncionario
 * Description: Camada de Dados
 * Copyright:    Copyright (c) 2002
 * Company: VIP Team
 * @author Thiago Arajo
 * @version 1.0
 */


public class RepositorioFuncionariosLista implements RepositorioFuncionarios {


    private Lista funcionarios;

    /**
     * Construtor padro.
    */
    public RepositorioFuncionariosLista() {
        funcionarios = new Lista();
    }

    /**
     * Insere um funcionrio.
     *
     * @param funcionario         funcionario a ser inserido.
     */
     public void inserir(Funcionario funcionario) {
        funcionarios.inserir(funcionario);
    }

    /**
     * Remove um funcionrio.
     *
     * @param funcionario         funcionario a ser removido.
     */
    public void remover(Funcionario funcionario) {
        funcionarios.remover(funcionario);
    }

    /**
     * Verifica se um funcionrio existe na lista.
     *
     * @param funcionrio         funcionario.
     * @return                    um boolean indicando a presena do funcionrio.
    */
    public boolean existe(Funcionario funcionario) {
        return funcionarios.existe(funcionario);
    }

    /**
     * Retorna os funcionrios contidos nessa lista.
     *
     * @return     "[]" caso a lista seja vazia. Caso contrrio retorna uma
     *             lista que comea com '[' e termina com ']' em que os
     *             elementos so separados por vrgula.
     * @see Funcionario
     */
    public String toString() {
        return funcionarios.toString();
    }

    /**
     * Retorna um RepositorioFuncionarios indicando os funcionrios que casam
     * com o funcionario fornecido de acordo com os parmetros passados.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @param campos    um inteiro indicando os campos (atributos) que sero
     *                  usados para comparar os funcionrios.
     *
     *                  Os critrios vlidos so MATCH_NOME, MATCH_TELEFONE,
     *                  MATCH_CIDADE, MATCH_BAIRRO, MATCH_CARGO, MATCH_LOGIN e
     *                  MATCH_SENHA. Vrios critrios podem
     *                  ser utilizados ao mesmo tempo usando OU lgico.
     *
     * @return          RepositorioFuncionarios com os Funcionrios que casam
     *                  com o dado.
     *
     * @see Funcionario
     * @see Lista
     *
     */
    public RepositorioFuncionarios match(Funcionario f, int campos) {
        RepositorioFuncionarios retorno;
//        Funcionario[] array = this.toArray();
        Funcionario temp;

        Iterator iterador = getIterator();

        retorno = new RepositorioFuncionariosLista();

        while (iterador.hasNext()) {
            temp = (Funcionario) iterador.next();
            if (f.match(temp,campos)) {
                retorno.inserir(temp);
            }
        }

/*        for (i = 0; i < array.length; i++) {
            if (f.match(array[i],campos)) {
                retorno.inserir(array[i]);
            }
        } */

        return retorno;
    }

    /**
     * Retorna um booleano indicando se um funcionrio faz um <i>like</i>
     * com outro.
     *
     * @param f         o funcionrio que ser comparado.
     *
     * @return          um boolean que diz se os funcionrios casam.
     *
    */
    public RepositorioFuncionarios like(Funcionario f) {
        RepositorioFuncionarios retorno;
        Funcionario temp;
        Iterator iterador = getIterator();

        retorno = new RepositorioFuncionariosLista();

        while (iterador.hasNext()) {
            temp = (Funcionario) iterador.next();
            if (temp.like(f)) {
                retorno.inserir(temp);
            }
        }

        return retorno;
    }

    /**
     * Retorna um array que representa o repositrio.
     *
     * @return um array com os funcionrios do repositrio.
    */
/*    public Funcionario[] toArray() {
        int i;
        Object[] array = funcionarios.toArray();
        Funcionario[] ret = new Funcionario[array.length];

        for (i = 0; i < array.length; i++) {
            ret[i] = (Funcionario) array[i];
        }

        return ret;
    } */

    /**
     * Atualiza um funcionrio no repositorio
     *
     * @param f     funcionrio que ser atualizado.
     */
    public void atualizar(Funcionario f) {
        RepositorioFuncionarios rep;
//        Funcionario[] arr;
        Iterator iterador;

        rep = this.match(f, Funcionario.MATCH_LOGIN);

        if (rep.eVazio() == true) {
            throw new IllegalArgumentException("funcionrio no encontrado");
        } else {
            iterador = rep.getIterator();
            ((Funcionario) iterador.next()).atualizar(f);
//            arr = rep.toArray();
//            arr[0].atualizar(f);
        }
    }

    /**
     * Verifica se o repositrio  vazio.
     *
     * @return um boolean indicando se o repositrio  vazio.
     */
    public boolean eVazio() {
        return this.funcionarios.eVazio();
    }

    /**
     * Retorna um iterador para a classe RepositorioFuncionariosLista
     *
     * @return    um iterador.
     */
    public Iterator getIterator() {
        return new IteratorArray(funcionarios);
    }

}