package pedido;


/**
 * Title: CadastroPedidos
 * Description: controla o cadastro de pedidos.
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author VIP Team
 * @version 1.0
 */

public class CadastroPedidos {


    private int numProxPedido = 0;
    private RepositorioPedidos pedidos;

    /**
     * Construtor.
     */
    public CadastroPedidos(RepositorioPedidos pedidos) {
        this.pedidos = pedidos;
    }


    /**
     * Lista os pedidos cadastrados.
     *
     * @return      uma String que representa o cadastro.
     */
    public String toString() {
        return pedidos.toString();
    }

    /**
     * Insere um pedido <b>novo</b> no cadastro.
     *
     * @param p     novo pedido.
     */
    public void inserir(Pedido p) {
        p.setNumero(getNumProxPedido());

        if (existe(p) == true) {
            throw new IllegalArgumentException("pedido j cadastrado");
        } else if (p.getProdutos().eVazio() == true) {
            throw new IllegalArgumentException("lista de produtos no pode " +
                                               "ser vazia");
        } else {
            pedidos.inserir(p);
        }
    }

    /**
     * Remove um pedido do cadastro.
     *
     * @param p     pedido a ser removido.
     */
    public void remover(Pedido p) {
        pedidos.remover(p);
    }

    /**
     * Verifica se um pedido existe no cadastro.
     *
     * @param p     pedido a ser verificado.
     * @return      um booleano indicando se o pedido est cadastrado.
     */
    public boolean existe(Pedido p) {
        return pedidos.existe(p);
    }

    /**
     * Retorna um repositorio de clientes que atendem aos critrios definidos.
     *
     * @param c         o cliente que ser usado como base da comparao.
     * @param criterios um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o cliente base com os clientes
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NUMERO,
     *                  MATCH_NOME_CLIENTE e MATCH_NOME_FUNCIONARIO.
     * @return          um repositorio com todos os clientes encontrados
     *                  que fazem <i>match</i> com o cliente base segundo os
     *                  critrios especificados.
     *
     * @see Cliente
     * @see RepositorioClientes
     */
    public RepositorioPedidos match(Pedido p, int criterios) {
        return pedidos.match(p,criterios);
    }

    /**
     * Atualiza um pedido no repositorio
     *
     * @param p     pedido que ser atualizado.
     */
    public void atualizar(Pedido p) {
        if (p.getProdutos().eVazio() == true) {
            throw new IllegalArgumentException("lista de produtos no pode " +
                                               "ser vazia");
        } else {
            this.pedidos.atualizar(p);
        }
    }

   /**
     * Retorna o nmero do prximo pedido.
     *
     * @return      o nmero que deve ser atribudo a um pedido novo.
     */
    public int getNumProxPedido() {
        return numProxPedido++;
    }

}