package pedido;

import util.Iterator;

/**
 * Title: interface RepositorioPedidos
 * Description: Agrupa todos os Pedidos
 * Copyright:    Copyright (c) 2002
 * Company: VIP team
 * @author Rafael Cavalcanti Ortolan
 * @version 1.0
 */

public interface RepositorioPedidos {

     /**
     *   Este metodo coloca mais um pedido na lista
     *
     * @param pedido    Sera utilizado para inclusao
     */
    public void inserir(Pedido pedido);

     /**
     *   Este metodo removeum pedido na lista
     *
     * @param pedido    Sera utilizado para remocao
     */
    public void remover(Pedido pedido);

     /**
     *   Este metodo procura um pedido na lista
     *
     * @param pedido    Sera utilizado para procurar o pedido
     */
    public boolean existe(Pedido pedido);

    /**
     * Retorna um repositrio com os pedidos que atendem aos critrios definidos
     *
     * @param p         o pedido que ser usado como base da comparao.
     * @param criterios  um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o pedido base com os pedidos
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_DESCRICAO,
     *                  MATCH_TAMANHO, MATCH_PRECO.
     * @return          um repositrio com os produtos achados.
     */
    public RepositorioPedidos match(Pedido pedido, int criterios);


    /**
     * Verifica se o repositrio  vazio.
     *
     * @return um boolean indicando se o repositrio  vazio.
     */
    public boolean eVazio();

     /**
     * Atualiza um pedido no repositorio
     *
     * @param p     pedido que ser atualizado.
     */
    public void atualizar(Pedido p);

    /**
     * Retorna um iterador para a classe RepositorioPedidos
     *
     * @return    um iterador.
     */
    public Iterator getIterator();
}