package pedido;

import util.Lista;
import util.Iterator;
import util.IteratorArray;

/**
 * Title: Class ListaPedidos
 * Description: Coloca os pedidos numa lista
 * Copyright:    Copyright (c) 2002
 * Company: VIP team
 * @author Rafael Cavalcanti Ortolan
 * @version 1.0
 */

public class RepositorioPedidosLista implements RepositorioPedidos {


    private Lista pedidos;

    /**
     *   Construtor da classe ListaPedidos
     */
    public RepositorioPedidosLista() {
        pedidos = new Lista();
    }

    /**
     *   Este metodo coloca mais um pedido na lista
     *
     * @param pedido    Sera utilizado para inclusao
     */
    public void inserir(Pedido pedido) {
        pedidos.inserir(pedido);
    }

     /**
     *   Este metodo remove um pedido da lista
     *
     * @param pedido    Sera utilizado para remocao
     */
    public void remover(Pedido pedido) {
        pedidos.remover(pedido);
    }

     /**
     *   Este metodo procura um pedido da lista
     *
     * @param pedido    Sera utilizado para procurar o pedido
     */
    public boolean existe(Pedido pedido) {
        return pedidos.existe(pedido);
    }

    /**
     * Retorna um RepositorioPedidos indicando os pedidos que casam
     * com o pedido fornecido de acordo com os parmetros passados.
     *
     * @param p         o pedido que ser comparado.
     *
     * @param criterios um inteiro indicando os campos (atributos) que sero
     *                  usados para comparar os pedidos.
     *
     *                  Os critrios vlidos so MATCH_NUMERO,
     *                  MATCH_NOME_CLIENTE e MATCH_NOME_FUNCIONARIO.
     *                  Vrios critrios podem ser utilizados
     *                  ao mesmo tempo usando OU lgico.
     *
     *
     * @return          RepositorioPedidos com os Pedidos que casam
     *                  com o dado.
     *
     * @see Funcionario
     * @see Lista
     *
     */
    public RepositorioPedidos match(Pedido p, int criterios) {
        RepositorioPedidos matches = new RepositorioPedidosLista();
        Pedido temp;
        Iterator iterador = getIterator();

        while (iterador.hasNext()) {
            temp = (Pedido) iterador.next();
            if (p.match(temp,criterios)) {
                matches.inserir(temp);
            }
        }

        return matches;
    }

    /**
     * Verifica se o repositrio  vazio.
     *
     * @return um boolean indicando se o repositrio  vazio.
     */
    public boolean eVazio() {
        return this.pedidos.eVazio();
    }

    /**
     * Retorna os pedidos contidos nessa lista.
     *
     * @return     "[]" caso a lista seja vazia. Caso contrrio retorna uma
     *             lista que comea com '[' e termina com ']' em que os
     *             elementos so separados por vrgula.
     * @see Pedido
     */
    public String toString() {
        return this.pedidos.toString();
    }

     /**
     * Atualiza um pedido no repositorio
     *
     * @param p     pedido que ser atualizado.
     */
    public void atualizar(Pedido p) {
        RepositorioPedidos rep;
        Iterator iterador;

        rep = this.match(p, Pedido.MATCH_NUMERO);

        if (rep.eVazio() == true) {
            throw new IllegalArgumentException("pedido no encontrado");
        } else {
            iterador = rep.getIterator();
            ((Pedido) iterador.next()).atualizar(p);
        }
    }

    /**
     * Retorna um iterador para a classe RepositorioPedidosLista
     *
     * @return    um iterador.
     */
    public Iterator getIterator() {
        return new IteratorArray(pedidos);
    }

}