package pessoa;

import util.DadosResidenciais;

/**
 * <p>Title: CadastroProdutos</p>
 * <p>Description: Classe que controla o cadastro de produtos</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * @author Rafael e Thiago
 * @version 1.0
 */

public abstract class Pessoa {


    private String nome;
    private DadosResidenciais dadosResidenciais;

    /**
     * Construtor padro da classe Pessoa.
     *
     * Constri uma pessoa padro.
     */
    public Pessoa() {
        nome = new String("");
        dadosResidenciais = new DadosResidenciais();
    }

    public Pessoa(String nome, DadosResidenciais dadosResidenciais) {
        setNome(nome);
        this.dadosResidenciais = dadosResidenciais;
    }

    /**
     *   Este metodo devolve o nome de uma pessoa
     *
     * @return    nome da pessoa
     */
    public String getNome() {
        return this.nome;
    }

    /**
     *   Este metodo atribui o nome da pessoa
     *
     * @param nome    Sera utilizado para a atribuicao
     */
    public void setNome(String nome) {
        if (!nome.equals("")) {
            this.nome = nome;
        } else {
            throw new IllegalArgumentException("Nome invlido");
        }
    }

    /**
     *   Este metodo devolve os dados residenciais da pessoa
     *
     * @return    dados residenciais da pessoa
     */
    public DadosResidenciais getDadosResidenciais() {
        return this.dadosResidenciais;
    }

    /**
     *   Este metodo atribui os dados residenciais da pessoa
     *
     * @param dadosResidenciais    Sera utilizado para a atribuicao
     */
    public void setDadosResidenciais(DadosResidenciais dadosResidenciais) {
        this.dadosResidenciais = dadosResidenciais;
    }

    /**
     * Compara duas pessoas.
     *
     * @param p     pessoa a ser comparada.
     * @return      um booleano indicando se as pessoas so equivalentes.
     */
    public abstract boolean equals(Object o);

    /**
     * Retorna um booleano indicando se uma pessoa faz um <i>match</i> com outra
     * de acordo com os critrios desejados.
     *
     * @return um booleano indicando se os pedidos fazem <i>match</i>.
     */
    public abstract boolean match(Pessoa p, int criterios);

    /**
     * Atualiza uma pessoa
     *
     * @param p     pessoa que ser atualizado.
     */
    public void atualizar(Pessoa p) {
        dadosResidenciais.atualizar(p.dadosResidenciais);
    }
}