package produto;

/**
 * <p>Title: Produto</p>
 * <p>Description: define a classe Produto.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * <p>Exemplo de uso:</p>
 * <pre>
 * Produto p = new Produto();
 * </pre>
 *
 * @author Mayerber Carvalho Neto
 * @version 1.0
 */

public class Produto {


    // constantes de tamanho.
    // pizza pequena
    public final static int TAMANH0_PEQUENO = 0;
    // pizza mdia
    public final static int TAMANHO_MEDIO = 1;
    // pizza grande
    public final static int TAMANHO_GRANDE = 2;
    // refrigerante em lata
    public final static int TAMANH0_LATA = 3;
    // refrigerante 2 litros
    public final static int TAMANH0_PET = 4;
    // strings com as descries das constantes:
    public static final String[] descricaoTamanho = {"Pequeno", "Mdio",
                                                    "Grande", "Lata",
                                                    "Dois litros"};

    // constantes de match.
    public static final int MATCH_NOME = 1;
    public static final int MATCH_DESCRICAO = 2;
    public static final int MATCH_TAMANHO = 4;
    public static final int MATCH_PRECO = 8;

    private String nome;
    private String descricao;
    private int tamanho;
    private float preco;

    /**
     * Construtor padro.
     *
     */
    public Produto() {
        this.nome = "<sem-nome>";
        this.descricao = "";
        this.tamanho = TAMANHO_MEDIO;
        this.preco = 0;
    }

    /**
     * Construtor.
     *
     * @param nome          nome do produto.
     * @param descricao     uma breve descrio sobre o produto.
     * @param tamanho       tamanho desse produto.
     * @param preco         preo do produto.
     *
     * @throws IllegalArgumentException se algum dos atributos estiver com
     * valor no aceitvel.
     */
    public Produto(String nome, String descricao, int tamanho, float preco)
            throws IllegalArgumentException {
        this.setNome(nome);
        this.setDescricao(descricao);
        this.setTamanho(tamanho);
        this.setPreco(preco);
    }

    /**
     * Retorna uma String que representa o produto.
     *
     * @return              uma String com o nome do produto seguido de seu
     *                      tamanho entre parntesis.
     */
    public String toString() {
        return this.nome + " (" + descricaoTamanho[this.tamanho] + ")";
    }

    /**
     * Altera o nome do produto.
     *
     * @param nome          nome do produto.
     *
     * @throws IllegalArgumentException caso o nome seja vazio.
     */
    public void setNome(String nome) throws IllegalArgumentException {
        if (nome.equals("")) {
            throw new IllegalArgumentException("Nome no pode ser vazio.");
        } else {
            this.nome = nome;
        }
    }

    /**
     * Retorna o nome do produto.
     *
     * @return              uma String contendo o nome do produto.
     */
    public String getNome() {
        return this.nome;
    }

    /**
     * Altera a descrio do produto.
     *
     * @param descricao     nova descrio para o produto.
     */
    public void setDescricao(String descricao) {
        this.descricao = descricao;
    }

    /**
     * Retorna a descrio do produto.
     *
     * @return              descrio do produto.
     */
    public String getDescricao() {
        return this.descricao;
    }

    /**
     * Altera o tamanho do produto.
     *
     * @param tamanho       um inteiro que representa o tamanho do produto.
     *
     * <p>Os tamanhos vlidos so:</p>
     * <p>TAMANH0_PEQUENO, TAMANH0_MEDIO, TAMANHO_GRANDE, TAMANH0_LATA e
     * TAMANH0_PET</p>
     *
     * @throws IllegalArgumentException caso o tamanho seja invlido.
     */
    public void setTamanho(int tamanho) throws IllegalArgumentException {
        if (tamanho != TAMANH0_PEQUENO && tamanho != TAMANHO_MEDIO
                && tamanho != TAMANHO_GRANDE && tamanho != TAMANH0_LATA
                && tamanho != TAMANH0_PET) {
            throw new IllegalArgumentException("Tamanho invlido.");
        } else {
            this.tamanho = tamanho;
        }
    }

    /**
     * Retorna o tamanho do produto.
     *
     * @return              tamanho do produto.
     */
    public int getTamanho() {
        return this.tamanho;
    }

    /**
     * Altera o preo do produto.
     *
     * @param preco     novo preo para o produto.
     *
     * @throws IllegalArgumentException caso o preo seja invldo.
     */
    public void setPreco(float preco) throws IllegalArgumentException {
        if (preco <= 0) {
            throw new IllegalArgumentException("Preo tem que ser maior "+
                                              "que zero.");
        } else {
            this.preco = preco;
        }
    }

    /**
     * Retorna o preo do produto.
     *
     * @return              preo do produto.
     */
    public float getPreco() {
        return this.preco;
    }

    /**
     * Compara dois produtos.
     *
     * <p>Os critrios de comparao utilizados so o <b>nome</b> e o
     * <b>tamanho</b>.</p>
     *
     * <p>A comparao do nome  <i>case-insensitive</i>.</p>
     *
     * @param o             produto a ser comparado.
     * @return              um booleano indicando se os produtos so
     *                      equivalentes.
     *
     * @throws ClassCastException caso o objeto passado no seja instncia de
     * Produto.
     */
    public boolean equals(Object o) throws ClassCastException {
        Produto p;

        if (o instanceof Produto) {
            p = (Produto) o;
            return (this.nome.equalsIgnoreCase(p.nome) && this.tamanho == p.tamanho);
        } else {
            throw new ClassCastException("No  instncia de Produto");
        }
    }

    /**
     * Retorna um booleano indicando se um produto faz um <i>match</i> com outro
     * de acordo com os critrios desejados.
     *
     * @param p         o produto que ser comparado.
     * @param criterios um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar esse produto (<i>this</i>) com o
     *                  produto fornecido.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_DESCRICAO,
     *                  MATCH_TAMANHO, MATCH_PRECO. Vrios critrios podem
     *                  ser utilizados ao mesmo tempo. Para isso, o valor
     *                  do parmetro <b>criterios</b> deve ser obtido atravs
     *                  do <b>OU lgico</b>. Por exemplo:
     *                  <br>
     *                  "criterios = MATCH_NOME | MATCH_PRECO"
     *                  </br>
     *                  significa que o nome e o preo dos produtos sero usados
     *                  como chaves de comparao.
     *
     * @return um booleano indicando se os produtos fazem <i>match</i>.
     */
    public boolean match(Produto p, int criterios) {
        boolean ok = true;

        if ((criterios & MATCH_NOME) == MATCH_NOME
                && !this.nome.equalsIgnoreCase(p.nome)) {
            ok = false;
        }

        if ((criterios & MATCH_DESCRICAO) == MATCH_DESCRICAO
                && !this.descricao.equalsIgnoreCase(p.descricao)) {
            ok = false;
        }

        if ((criterios & MATCH_TAMANHO) == MATCH_TAMANHO
               && this.tamanho != p.tamanho) {
            ok = false;
        }

        if ((criterios & MATCH_PRECO) == MATCH_PRECO
                && this.preco != p.preco) {
            ok = false;
        }

        return ok;
    }

     /**
     * Atualiza um produto
     *
     * @param p     produto que ser atualizado.
     *
     * @throws IllegalArgumentException caso o valor de algum dos atributos no
     * seja aceitvel.
     */
    public void atualizar(Produto p) throws IllegalArgumentException {
        this.setDescricao(p.descricao);
        this.setPreco(p.preco);
    }

    public static void main(String[] args) {
        Produto p = new Produto();
        Produto p2 = new Produto("Mussarela","queijo, tomate, etc.",
                                 TAMANHO_GRANDE,10.0f);
        Produto p3 = new Produto("Mussarela","queijo, tomate, etc.",
                                 TAMANHO_MEDIO,11.0f);
        Produto p4 = new Produto("Calabresa","queijo, tomate, calabresa, etc.",
                                 TAMANHO_MEDIO,12.0f);
        Produto p5 = new Produto("Coca-Cola","Coca-Cola",TAMANH0_LATA,1.0f);

        System.out.println(p);
        System.out.println(p2);
        System.out.println(p3);
        System.out.println(p4);
        System.out.println("p2 faz match de NOME com MUSSARELA? " +
                            p2.match(p3,MATCH_NOME));
        System.out.println("p2 faz match de TAMANHO com MUSSARELA? " +
                            p2.match(p3,MATCH_TAMANHO));
        System.out.println("p2 faz match de NOME e TAMANHO com p2? " +
                            p2.match(p2,MATCH_NOME | MATCH_TAMANHO));
    }

}