package produto;

import util.Iterator;


/**
 * <p>Title: RepositorioProdutos</p>
 * <p>Description: Interface para o repositrio de produtos</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * @author Mayerber Carvalho Neto
 * @version 1.0
 */

public interface RepositorioProdutos {


    /**
     * Insere um produto
     *
     * @param p produto
     */
    public void inserir(Produto p);

    /**
     * Remove um produto
     *
     * @param p produto
     */
    public void remover(Produto p);

    /**
     * Verifica se um produto existe
     *
     * @param p produto
     * @return  um booleano indicando a existncia do produto.
     */
    public boolean existe(Produto p);

    /**
     * Verifica se o repositrio  vazio.
     *
     * @return um boolean indicando se o repositrio  vazio.
     */
    public boolean eVazio();

    /**
     * Retorna um repositrio com os produtos que atendem aos critrios
     * definidos.
     *
     * @param p         o produto que ser usado como base da comparao.
     * @param criterios  um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o produto base com os produtos
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_DESCRICAO,
     *                  MATCH_TAMANHO, MATCH_PRECO.
     * @return          um repositrio com os produtos achados.
     */
    public RepositorioProdutos match(Produto p, int criterios);

     /**
     * Atualiza um produto no repositorio
     *
     * @param p     produto que ser atualizado.
     */
    public void atualizar(Produto p);

    /**
     * Retorna um iterador para a classe RepositorioProdutos
     *
     * @return    um iterador.
     */
    public Iterator getIterator();
}