package produto;

import util.Lista;
import util.Iterator;
import util.IteratorArray;


/**
 * <p>Title: RepositorioProdutosLista</p>
 * <p>Description: Implementa uma lista de produtos</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * @author Mayerber Carvalho Neto
 * @version 1.0
 */

public class RepositorioProdutosLista implements RepositorioProdutos {


    private Lista produtos;

    /**
     * Construtor padro.
     */
    public RepositorioProdutosLista() {
        this.produtos = new Lista();
    }

    /**
     * Retorna os produtos contidos nessa lista.
     *
     * @return     "[]" caso a lista seja vazia. Caso contrrio retorna uma
     *             lista que comea com '[' e termina com ']' em que os
     *             elementos so separados por vrgula.
     * @see Produto
     */
    public String toString() {
        return this.produtos.toString();
    }

    /**
     * Insere um produto.
     *
     * @param p         produto a ser inserido.
     */
    public void inserir(Produto p) {
        this.produtos.inserir(p);
    }

    /**
     * Remove um produto da lista. Tal produto <b>deve</b> constar na lista.
     *
     * @param p         produto a ser removido.
     */
    public void remover(Produto p) {
        this.produtos.remover(p);
    }

    /**
     * Verifica se um produto existe na lista.
     *
     * @param p         produto.
     * @return          um booleano indicando a presena do produto na lista.
     */
    public boolean existe(Produto p) {
        return this.produtos.existe(p);
    }

    /**
     * Retorna uma lista de produtos que atendem aos critrios definidos.
     *
     * @param p         o produto que ser usado como base da comparao.
     * @param criterios um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o produto base com os produtos
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_DESCRICAO,
     *                  MATCH_TAMANHO, MATCH_PRECO.
     *
     * @return um repositorio de produtos que atendem aos critrios definidos.
     * @see Produto
     * @see Lista
     */
    public RepositorioProdutos match(Produto p, int criterios) {
        RepositorioProdutos matches = new RepositorioProdutosLista();
        Produto temp;
        Iterator iterador = getIterator();

        while (iterador.hasNext()) {
            temp = (Produto) iterador.next();
            if (p.match(temp,criterios)) {
                matches.inserir(temp);
            }
        }

        return matches;
    }

    /**
     * Verifica se o repositrio  vazio.
     *
     * @return um boolean indicando se o repositrio  vazio.
     */
    public boolean eVazio() {
        return this.produtos.eVazio();
    }

     /**
     * Atualiza um produto no repositorio
     *
     * @param p     produto que ser atualizado.
     */
    public void atualizar(Produto p) {
        RepositorioProdutos rep;
        Iterator iterador;

        rep = this.match(p,Produto.MATCH_NOME | Produto.MATCH_TAMANHO);

        if (rep.eVazio() == true) {
            throw new IllegalArgumentException("produto no encontrado");
        } else {
            iterador = rep.getIterator();
            ((Produto) iterador.next()).atualizar(p);
        }
    }

    /**
     * Retorna um iterador para a classe RepositorioProdutosLista
     *
     * @return    um iterador.
     */
    public Iterator getIterator() {
        return new IteratorArray(produtos);
    }

    public static void main(String[] args) {
        Produto procura;
        RepositorioProdutosLista lista = new RepositorioProdutosLista();
        Produto p1 = new Produto("Mussarela","queijo, tomate, etc.",
                                 Produto.TAMANHO_MEDIO,11.0f);
        Produto p2 = new Produto("Calabresa","queijo, tomate, calabresa, etc.",
                                 Produto.TAMANHO_MEDIO,12.0f);
        Produto p3 = new Produto("pizza grande genrica","hehe",
                                 Produto.TAMANHO_GRANDE,10.0f);

        lista.inserir(p1);
        System.out.println(lista);
        lista.inserir(p2);
        System.out.println(lista);
        lista.inserir(p3);
        System.out.println(lista);
        System.out.println("produtos com tamanho GRANDE: " +
                            lista.match(p3,Produto.MATCH_TAMANHO));
    }

}