package util;


/**
 * <p>Title: Lista</p>
 * <p>Description: Classe que implementa uma lista genrica</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 */

public class Lista {


    private Object no;
    private Lista prox;
    private int tamanho;

    /**
     * Construtor padro da classe Lista.
     *
     * <p>Constri uma lista vazia.</p>
     */
     public Lista() {
        this.no = null;
        this.prox = null;
        this.tamanho = 0;
    }

    /**
     * Retorna uma String que representa os elementos (objetos) contidos na
     * lista. Cada elemento  convertido para String atravs do seu mtodo
     * <b>toString()</b>.
     *
     * @return     "[]" caso a lista seja vazia. Caso contrrio retorna uma
     *             lista que comea com '[' e termina com ']' em que os
     *             elementos so separados por vrgula.
     */
    public String toString() {
        String s = "[";
        Lista l = this;

        while (l != null) {
            if (l.no != null) {
                if (s != "[") {
                    s = s + ", " + l.no;
                } else {
                    s = s + l.no;
                }
            }

            l = l.prox;
        }

        s= s + "]";
        return s;
    }

    /**
     * Insere um elemento na lista.
     *
     * @param o     objeto a ser inserido.
     */
    public void inserir(Object o) {
        this.tamanho++;

        if (this.no == null) {
            this.no = o;
            this.prox = new Lista();
        } else {
            this.prox.inserir(o);
        }
    }

    /**
     * Remove um objeto da lista.
     *
     * @param o     objeto a ser removido.
     * @return  1 (um) se o objeto foi removido, 0 (zero) caso contrrio.
     */
    public int remover(Object o) {
        int ret;

        if (this.no == null) {
            ret = 0;
        } else if (this.no.equals(o)) {
            this.no = prox.no;
            this.prox = prox.prox;
            ret = 1;
        } else {
            ret = this.prox.remover(o);
        }

        // atualiza o tamanho da lista:
        this.tamanho -= ret;

        return ret;
    }

    /**
     * Verifica se um objeto pertence  lista.
     *
     * @param o     objeto a ser procurado.
     * @return      um booleano indicando a presena ou no do objeto.
     */
    public boolean existe(Object o) {
        boolean ret;

        if (this.no == null) {
            ret = false;
        } else if (this.no.equals(o)) {
            ret = true;
        } else {
            ret = this.prox.existe(o);
        }

        return ret;
    }

    /**
     * Retorna o tamanho (nmero de elementos) da lista.
     *
     * @return      um inteiro indicando a quantidade de elementos presentes
     *              na lista.
     */
    public int getTamanho() {
        return this.tamanho;
    }

    /**
     * Remove todos os elementos da lista.
     */
    public void limpar() {
        this.no = null;
        this.prox = null;
        this.tamanho = 0;
    }

    /**
     * Verifica se a lista  vazia.
     *
     * @return          um booleano indicando se a lista  vazia.
     */
    public boolean eVazio() {
        boolean ret;

        if (this.tamanho == 0) {
            ret = true;
        } else {
            ret = false;
        }

        return ret;
    }

    /**
     * Retorna o elemento armazenado nesse n.
     *
     * @return          o objeto.
     */
    public Object getNo() {
        return no;
    }

    /**
     * Retorna a prxima lista.
     *
     * @return          prxima lista.
     */
    public Lista getProx() {
        return prox;
    }

    public static void main(String[] args) {
        Lista lista1 = new Lista();

        System.out.println("lista criada...");
        System.out.println("num. de elementos = " + lista1.getTamanho());

        System.out.println("inserindo elem. 0");
        lista1.inserir(new Integer(0));
        System.out.println(lista1);
        System.out.println("num. de elementos = " + lista1.getTamanho());

        System.out.println("inserindo 0, 1, 2, 3");
        lista1.inserir(new Integer(0));
        lista1.inserir(new Integer(1));
        lista1.inserir(new Integer(2));
        lista1.inserir(new Integer(3));
        System.out.println(lista1);

        System.out.println("limpando a lista...");
        lista1.limpar();
        System.out.println(lista1);

        System.out.println("inserindo 0");
        lista1.inserir(new Integer(0));
        System.out.println(lista1);

        System.out.println("inserindo 2, 4, 8");
        lista1.inserir(new Integer(2));
        lista1.inserir(new Integer(4));
        lista1.inserir(new Integer(8));
        System.out.println(lista1);
        System.out.println("num. de elementos = " + lista1.getTamanho());

        System.out.println("removendo elem. inexistente 100...");
        lista1.remover(new Integer(100));
        System.out.println(lista1);
        System.out.println("num. de elementos = " + lista1.getTamanho());

        System.out.println("removendo elem. 2 e 4");
        lista1.remover(new Integer(2));
        lista1.remover(new Integer(4));
        System.out.println(lista1);
        System.out.println("num. de elementos = " + lista1.getTamanho());

        System.out.println("removendo elem. 8");
        lista1.remover(new Integer(8));
        System.out.println(lista1);
        System.out.println("num. de elementos = " + lista1.getTamanho());


        System.out.println("lista eh vazia? " + lista1.eVazio());

    }

}
